#include "Parametric.h"

#include <GL/glut.h>
#include <cmath>

Vector3 Parametric::f(float u, float v)
{
	const float R = 5;
	const float r = 1;
	const float h = 2;

	Vector3 vec;
	vec.x = (R+r*cos(u))*cos(v);
	vec.y = (R+r*cos(u))*sin(v);
	vec.z = r*sin(u)+(v-PI)*h;

	return vec;
}

Parametric::Parametric(int u_res, int v_res) :
	N(u_res), M(v_res),
	UMax(2*PI), VMax(2*PI)
{
	vx = new Vector3[N*M];
	nx = new Vector3[N*M];

	const float UStep = UMax/N;
	const float VStep = VMax/M;

	int idx = 0;

	for ( float v=0; v<VMax; v += VStep )
		for ( float u=0; u<UMax; u += UStep )
		{
			vx[idx] = f(u,v);
			nx[idx] = -(f(u-UStep,v)-f(u+UStep,v)).crossProduct(f(u,v-VStep)-f(u,v+VStep));
			nx[idx].normalise();
			++idx;
		}
}

Parametric::~Parametric(void)
{
	delete [] vx;
	delete [] nx;
}

void Parametric::Draw(void)
{
	glBegin(GL_QUADS);
		glColor3f(0.7f, 0.7f, 0.2);
		for ( int v=0; v<M-1; ++v )
			for ( int u=0; u<N; ++u )
			{
				const int UNext = (u+1)%N;
				const int VNext = (v+1)%M;

				glNormal3fv(nx[v*N+u].ptr());
				glVertex3fv(vx[v*N+u].ptr());

				glNormal3fv(nx[VNext*N+u].ptr());
				glVertex3fv(vx[VNext*N+u].ptr());

				glNormal3fv(nx[VNext*N+UNext].ptr());
				glVertex3fv(vx[VNext*N+UNext].ptr());

				glNormal3fv(nx[v*N+UNext].ptr());
				glVertex3fv(vx[v*N+UNext].ptr());
			}
	glEnd();
}