#include <GL/glut.h>

#include "Parametric.h"

float rtspd = 0;
float rot = 0;
float rad = 2;

float pX, pY;
bool dragging = false;

Parametric ize(10, 50);

struct camera
{
    camera(float x=0, float y=0, float z=0, float tPitch=0, float tYaw=0) : pitch(tPitch), yaw(tYaw)
    {
        pos[0] = x; pos[1] = y; pos[2] = z;
    }

    void setView()
    {
        glRotatef(pitch, 1,0,0);
        glRotatef(yaw, 0,0,1);
        glTranslatef(-pos[0], -pos[1], -pos[2]);
    }

    float pos[3];
    float pitch, yaw;
};

camera cam(0, -12, 5, -75, 0);

void setLights()
{
    static const float lightAmbient1[]= { 0.1f, 0.1f, 0.1f, 1.0f };
    static const float lightDiffuse1[]= { 2.0f, 2.0f, 2.0f, 1.0f };

    glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient1);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse1);
    glEnable(GL_LIGHT1);

    static const float lightAmbient2[]= { 0.1f, 0.1f, 0.1f, 1.0f };
    static const float lightDiffuse2[]= { 0.0f, 0.0f, 5.0f, 1.0f };

    glLightfv(GL_LIGHT2, GL_AMBIENT, lightAmbient2);
    glLightfv(GL_LIGHT2, GL_DIFFUSE, lightDiffuse2);
    glEnable(GL_LIGHT2);

	glEnable(GL_LIGHTING);

	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
}

void setLightsPos()
{
    static const float lightPosition1[]= { 8.0f, 0.0f, 0.0f, 1.0f };
	static const float lightDirection1[]= { -1.0f, 0.0f, 0.0f };

    glLightfv(GL_LIGHT1, GL_POSITION,lightPosition1);
	glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION,lightDirection1);


    static const float lightPosition2[]= { -8.0f, 0.0f, 0.0f, 1.0f };
	static const float lightDirection2[]= { 1.0f, 0.0f, 0.0f };

    glLightfv(GL_LIGHT2, GL_POSITION,lightPosition2);
	glLightfv(GL_LIGHT2, GL_SPOT_DIRECTION,lightDirection2);
}

void keyboard(unsigned char key, int x, int y)
{
    switch (key)
    {
    case '+':
        rad += 2;
        break;
    case '-':
        rad -= 2;
        break;
    case '*':
        rtspd += 0.5f;
        break;
    case '/':
        rtspd -= rtspd > 0 ? 0.5f : 0;
        break;
    case 'w':
        cam.pos[1] += 1;
        break;
    case 's':
        cam.pos[1] -= 1;
        break;
    case 'a':
        cam.pos[0] -= 1;
        break;
    case 'd':
        cam.pos[0] += 1;
        break;
    case 'r':
        cam.pos[2] += 1;
        break;
    case 'f':
        cam.pos[2] -= 1;
        break;

    }
}

void mouseMove(int x, int y)
{
    if (dragging)
    {
        cam.yaw += (x-pX)/10;
        cam.pitch += (y-pY)/10;
        pX = static_cast<float>(x);
        pY = static_cast<float>(y);
    }
}

void mouseClick(int button, int state, int x, int y)
{
    if (button == GLUT_LEFT_BUTTON)
    {
        pX = static_cast<float>(x);
        pY = static_cast<float>(y);
        dragging = (state == GLUT_DOWN);
    }
}

void reshape(int width, int height)
{
    glViewport(0,0,width,height);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45, 1.0f*width/height, 0.1f, 200);
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    cam.setView();

	setLightsPos();

	glPushMatrix();

	glTranslatef(0,rad,0);
    glRotatef(rot, 0,0,1);
    rot += rtspd;

	ize.Draw();

    glPopMatrix();

    glutSwapBuffers();
}

int main(int argc, char **argv)
{
    glutInit(&argc, argv);

    glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
    glutCreateWindow("Parametric");

    glClearDepth(1.0f);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);

    glutDisplayFunc(display);
    glutIdleFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouseClick);
	glutMotionFunc(mouseMove);

	setLights();

    glutMainLoop();
    return 0;
}
